#!/usr/bin/env python
"""Setup script for mcp-ticketer package.

This file is provided for backward compatibility with older pip versions
and tools that don't support PEP 517/518 build systems.
"""

from setuptools import setup

# The actual package configuration is in pyproject.toml
# This file just calls setup() to maintain compatibility
if __name__ == "__main__":
    setup()
